import { DataService } from './../../services/data.service';
import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { take } from "rxjs/operators";

@Component({
  selector: 'app-class-page',
  templateUrl: './class-page.component.html',
  styleUrls: ['./class-page.component.css']
})
export class ClassPageComponent implements OnInit {

  classes = [];
  filteredClasses = [];// Clases del día marcado
  monthNames = ["Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio",
    "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre"
  ];
  date = new Date();// Fecha de hoy o seleccionada
  actualWeek = true;// Selecciona esta semana o la siguiente

  constructor(private httpClient: HttpClient,
    public dataService: DataService) { }

  ngOnInit(): void {
    this.getClasses();
  }

  // Obtiene las clases y las añades a this.classes
  private getClasses() {
    this.classes = [];
    this.httpClient.get(this.dataService.server + "classes?user_uid=" +
      this.dataService.session.userUid)
      .pipe(take(1))
      .subscribe((classes: any[]) => {
        if (classes && classes.length) {
          for (let i = 0; i < classes.length; i++) {
            const element = classes[i];
            element.day = new Date(element.day);
            this.classes.push(element);
          }
          this.updateFilteredClasses()
        }
      })
  }

  public selectedDayClass(dayNumber: number) {
    if (dayNumber == this.date.getDay())
      return true;
    else return false;
  }

  // Nombre de la fecha de hoy o la que esté seleccionada
  public getDateName() {
    return this.date.getDate() + " " + this.monthNames[this.date.getMonth()] + " " +
      this.date.getFullYear()
  }

  // Calcula el cambio de fecha
  public changeDate(dayNumber: number) {
    if (!this.disabledDayClass(dayNumber)) {

      let daySelected = this.date.getDay();
      if (daySelected == 0) {
        daySelected = 7;
      }

      if (daySelected != dayNumber) {
        let change = dayNumber - daySelected;
        this.date = new Date(this.date.getFullYear(), this.date.getMonth(),
          this.date.getDate() + change);
        this.updateFilteredClasses();
      }
    }
  }

  public disabledDayClass(dayNumber: number) {
    let now = new Date();
    if (this.actualWeek == true) {
      return now.getDay() > dayNumber
    }
    return false;
  }

  // Suma o resta una semana
  public changeWeek(newActualWeek: boolean) {
    if (newActualWeek) {
      this.date = new Date();
      this.updateFilteredClasses();
    } else {
      this.date = new Date(this.date.getFullYear(), this.date.getMonth(),
        this.date.getDate() + 7);
      this.updateFilteredClasses();
    }
    this.actualWeek = newActualWeek;
  }

  private updateFilteredClasses() {
    const date00h = new Date(this.date.getFullYear() + "-" + (this.date.getMonth() + 1) + "-" + this.date.getDate());
    const date24h = new Date(date00h.getTime() + (86400 * 1000));

    this.filteredClasses = this.classes.filter(element => element.day &&
      element.day.getTime() > date00h.getTime() && element.day.getTime() < date24h.getTime());
    this.filteredClasses = this.filteredClasses.sort((a, b) => a.day.getTime() - b.day.getTime());
  }

  public getStartHour(element: any) {
    const hours = element.day.getHours().toString();
    if (hours.length == 1) {
      return "0" + hours;
    } else {
      return hours;
    }
  }

  public addToClass(element) {
    if (this.dataService.session.logged) {
      this.httpClient.post(this.dataService.server + "classes/update", {
        class: element,
        user_uid: this.dataService.session.userUid
      })
        .subscribe((result: any) => {
          this.getClasses();
        })
    }
  }

}
