import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { DataService } from 'src/app/services/data.service';
import { take } from 'rxjs/operators';

@Component({
  selector: 'app-machine-page',
  templateUrl: './machine-page.component.html',
  styleUrls: ['./machine-page.component.css']
})
export class MachinePageComponent implements OnInit {

  table: any;//Tabla completa guardada
  machinesSavedThisDay: any;//Tabla del día seleccionado guardada
  machinesThisDay: any;//Máquinas del día seleccionado

  machines: any;
  monthNames = ["Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio",
    "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre"
  ];
  weekDays = ["D", "L", "M", "X", "J", "V"];
  date = new Date();// Fecha de hoy o seleccionada
  actualWeek = true;// Selecciona esta semana o la siguiente

  edit = false;// Editar la tabla
  dayOfWeek;

  constructor(private httpClient: HttpClient,
    public dataService: DataService) { }

  ngOnInit(): void {
    this.getAllMachines();
  }

  private getTable() {
    this.httpClient.get(this.dataService.server + "table?user_uid=" + this.dataService.session.userUid)
      .pipe(take(1))
      .subscribe((table: any) => {
        if (table) {
          this.table = table;
        } else {
          this.table = {
            days: [JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines)),
            JSON.parse(JSON.stringify(this.machines))]
          }
        }
        this.updateMachinesByDay();
      })
  }

  private getAllMachines() {
    this.httpClient.get(this.dataService.server + "machines")
      .pipe(take(1))
      .subscribe((machines: any[]) => {
        this.machines = machines;
        this.getTable();
      })
  }

  public selectMachine(machine: any) {
    machine.selected = !machine.selected;
  }

  public selectedDayClass(dayNumber: number) {
    if (dayNumber == this.date.getDay())
      return true;
    else return false;
  }

  // Nombre de la fecha de hoy o la que esté seleccionada
  public getDateName() {
    return this.date.getDate() + " " + this.monthNames[this.date.getMonth()] + " " +
      this.date.getFullYear()
  }

  // Calcula el cambio de fecha
  public changeDate(dayNumber: number) {
    let daySelected = this.date.getDay();
    if (daySelected == 0) {
      daySelected = 7;
    }

    if (daySelected != dayNumber) {
      let change = dayNumber - daySelected;
      this.date = new Date(this.date.getFullYear(), this.date.getMonth(),
        this.date.getDate() + change);
      this.updateMachinesByDay();
    }
  }

  updateMachinesByDay() {
    if (this.table && this.table.days) {
      this.machinesThisDay = this.table.days[this.date.getDay()];
      this.machinesSavedThisDay = {
        [Categories.ARMS]: this.filterByToday(this.table.days[this.date.getDay()], Categories.ARMS),
        [Categories.BACK]: this.filterByToday(this.table.days[this.date.getDay()], Categories.BACK),
        [Categories.CHEST]: this.filterByToday(this.table.days[this.date.getDay()], Categories.CHEST),
        [Categories.LEGS]: this.filterByToday(this.table.days[this.date.getDay()], Categories.LEGS),
        [Categories.SHOULDERS]: this.filterByToday(this.table.days[this.date.getDay()], Categories.SHOULDERS)
      }
    }
  }

  //Filtra las máquinas por las que tiene en la tabla y el día
  public filterByToday(machines: any, category: Categories) {
    if (machines && machines[category]) {
      let filtered = machines[category].filter(x => this.machinesThisDay[category].findIndex(y => y.id == x.id && y.selected) != -1);

      return filtered;
    } else {
      return [];
    }
  }

  public cancel() {
    this.edit = false;
    this.getTable();
  }

  public srcButtonImage() {
    let src = "/assets/icons/";
    if (!this.edit) {
      src += "edit.png";
    } else {
      src += "save.png";
    }
    return src;
  }

  public editOrSave() {
    if (this.edit) {
      this.httpClient.post(this.dataService.server + "table/update",
        {
          user_uid: this.dataService.session.userUid,
          table: this.table
        })
        .subscribe(() => {
          this.edit = false;
          this.updateMachinesByDay()
        })
    } else {
      this.edit = true;
    }
  }
}

enum Categories {
  "SHOULDERS" = "shoulders",
  "LEGS" = "legs",
  "BACK" = "back",
  "CHEST" = "chest",
  "ARMS" = "arms"
}
