import { DataService } from './../../services/data.service';
import { HttpClient } from '@angular/common/http';
import { Router } from '@angular/router';
import { Component, OnInit } from '@angular/core';

@Component({
  selector: 'app-request',
  templateUrl: './request.component.html',
  styleUrls: ['./request.component.css']
})
export class RequestComponent implements OnInit {

  url: string;
  title: string;
  description: string;
  type: string;
  request = {
    information: "",
    options: {}
  }

  constructor(private router: Router, private httpClient: HttpClient,
    private dataService: DataService) { }

  ngOnInit(): void {
    this.getUrl();
  }

  private getUrl() {
    this.url = this.router.url;
    switch (this.url) {
      case "/solicitar-tabla":
        this.title = "Solicitar tabla";
        this.description = "";
        this.type = "TABLA";
        break;
      case "/solicitar-ayuda":
        this.title = "Solicitar ayuda";
        this.description = "Por favor, indíquenos el problema y dónde se encuentra";
        this.type = "AYUDA";
        break;
      case "/solicitar-informacion":
        this.title = "Solicitar información";
        this.description = "Por favor, indíquenos qué información necesita";
        this.type = "INFORMATION";
        break;
    }
  }

  public send() {
    this.httpClient.post(this.dataService.server + "request/create", {
      user_uid: this.dataService.session.userUid,
      request: this.request,
      type: this.type
    }).subscribe(() => {
      this.request = {
        information: "",
        options: {
          type: null
        }
      }
      alert("Información enviada correctamente");
    })
  }

}
