import { DataService } from './../../services/data.service';
import { Router } from '@angular/router';
import { Component, OnInit } from '@angular/core';
import { AngularFireAuth } from '@angular/fire/compat/auth';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit {

  showLogin = false;
  email = '';
  password = '';
  showRegister = false;
  registerFirstName = '';
  registerLastName = '';
  registerEmail = '';
  registerPassword = '';
  loadPage = false; // Cargando por si hay credenciales ya

  constructor(private router: Router, private dataService: DataService,
    private fireAuth: AngularFireAuth) {
    this.dataService.session.guest = false;
    this.dataService.session.logged = false;
  }

  ngOnInit(): void {
    this.checkLogin();
  }

  public showLoginForm() {
    this.showLogin = true;
    this.email = '';
    this.password = '';
  }

  public redirectGuest() {
    this.dataService.session.guest = true;
    this.router.navigateByUrl("/home");
  }

  public async login() {
    if (this.email != '' && this.password != '') {
      if (!this.validPassword(this.password)) {
        this.showError("La contraseña debe tener, al menos, 6 caracteres");
      } else if (!this.validEmail(this.email)) {
        this.showError("Email inválido");
      } else {
        try {
          let result = await this.fireAuth.signInWithEmailAndPassword(this.email, this.password);
          if (result != null) {
            this.access(result, result.user.displayName, this.email, this.password);
          } else {
            this.showError();
          }
        } catch (err) {
          this.showError();
          console.error(err);
        }
      }
    } else {
      alert('Para acceder, incluya usuario y contraseña');
    }
  }

  public showRegisterForm() {
    this.showLogin = false;
    this.showRegister = true;
  }

  public async register() {
    if (this.registerFirstName != '' && this.registerLastName != ''
      && this.registerEmail != '' && this.registerPassword != '') {
      if (!this.validPassword(this.registerPassword)) {
        this.showError("La contraseña debe tener, al menos, 6 caracteres");
      } else if (!this.validEmail(this.registerEmail)) {
        this.showError("Email inválido");
      } else {
        try {
          let result = await this.fireAuth.createUserWithEmailAndPassword(
            this.registerEmail, this.registerPassword);
          if (result != null) {
            const completeName = this.registerFirstName + " " + this.registerLastName;
            result.user.updateProfile({
              displayName: completeName
            })
              .then(() => {
                this.access(result, completeName, this.registerEmail, this.registerPassword);
              }).catch(() => {
                this.showError();
              })
          } else {
            this.showError();
          }
        } catch (err) {
          this.showError();
          console.error(err);
        }
      }
    } else {
      this.showError('Para acceder, rellene todos los campos');
    }
  }

  // Alerta con error
  private showError(message?: string) {
    if (message) {
      alert(message);
    } else {
      alert("Ha ocurrido un error. Por favor, revise los datos e inténtelo de nuevo");
    }
  }

  private validPassword(password: string): boolean {
    return password.length > 5;
  }

  private validEmail(email: string): boolean {
    return email.match(
      /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
    ) != null;
  }

  //Accede a la aplicación
  private access(result, completeName, email, password) {
    this.dataService.session.name = completeName;
    this.dataService.session.logged = true;
    this.dataService.session.userUid = result.user.uid;
    localStorage.setItem('mind-muscle-e', email);
    localStorage.setItem('mind-muscle-p', password);
    this.router.navigateByUrl("/home");
  }

  // En localstorage puede tener los datos del login
  private async checkLogin() {
    let email = localStorage.getItem('mind-muscle-e');
    let password = localStorage.getItem('mind-muscle-p');
    if(email != null && password != null) {
      this.email = email;
      this.password = password;
      await this.login();
      this.loadPage = true;
    } else {
      this.loadPage = true;
    }
  }

}
