const express = require("express");
const cors = require('cors');
const db = require('./firebase/config');
const app = express();
const port = process.env.PORT || 3000;

app.use(express.json());
app.use(cors());

const Classes = db.collection("clases");
const ArmsMachine = db.collection("maquinas-brazo");
const BackMachine = db.collection("maquinas-espalda");
const ShouldersMachine = db.collection("maquinas-hombro");
const ChestMachine = db.collection("maquinas-espalda");
const LegsMachine = db.collection("maquinas-pierna");
const Tables = db.collection("tablas");
const Request = db.collection("solicitudes");

app.get("/", (req, res) => {
    res.send("Servidor de Mind Muscle");
});


// Servicios:
// Si se manda user_uid lo busca por ese dato
app.get("/classes", async (req, res) => {
    const userUid = req.query.user_uid;
    const allData = await Classes.get();
    const result = allData.docs.map((doc) => ({ id: doc.id, ...doc.data() }));
    if (result && result.length) {
        for (let i = 0; i < result.length; i++) {
            let usersRegistered = []; // Usuarios inscritos a la clase
            if (usersRegistered && result[i].user_uid) {
                usersRegistered = JSON.parse(result[i].user_uid);
            }

            if (userUid) {
                if (usersRegistered.findIndex(x => x == userUid) != -1) {
                    result[i].registered = true;
                } else {
                    result[i].registered = false;
                }
            }
            result[i].users_registered = usersRegistered.length;
        }
    }
    res.send(result);
});

app.post('/classes/create', async (req, res) => {
    await Classes.add(req.body);
    res.send({ msg: "Class created" });
});

app.post('/classes/update', async (req, res) => {
    let element = req.body.class;
    let uid = req.body.user_uid;
    delete element.users_registered;
    delete element.registered;
    if (element.user_uid) {
        let users = JSON.parse(element.user_uid);
        let index = users.findIndex(x => x == uid);
        if (index == -1) {
            users.push(uid);
        } else {
            users.splice(index, 1);
        }
        element.user_uid = JSON.stringify(users);
    } else {
        element.user_uid = JSON.stringify([uid]);
    }

    await Classes.doc(element.id).update(element);
    res.send({ msg: "Class updated" });
});

// Devuelve todas las máquinas
app.get("/machines", async (req, res) => {
    let arms = await ArmsMachine.get();
    let back = await BackMachine.get();
    let shoulders = await ShouldersMachine.get();
    let chest = await ChestMachine.get();
    let legs = await LegsMachine.get();
    let machines = {
        arms: arms.docs.map((doc) => ({ id: doc.id, ...doc.data() })),
        back: back.docs.map((doc) => ({ id: doc.id, ...doc.data() })),
        shoulders: shoulders.docs.map((doc) => ({ id: doc.id, ...doc.data() })),
        chest: chest.docs.map((doc) => ({ id: doc.id, ...doc.data() })),
        legs: legs.docs.map((doc) => ({ id: doc.id, ...doc.data() }))
    };

    res.send(machines);
});

// Devuelve la tabla de un usuario
app.get("/table", async(req, res) => {
   const userUid = req.query.user_uid;
   let tableData = await (await Tables.doc(userUid).get()).data();
   res.send(tableData);
});

// Actualiza la tabla de un usuario
app.post("/table/update", async (req, res) => {
    const userUid = req.body.user_uid;
    await Tables.doc(userUid).set(req.body.table);

    res.send({ msg: "Table updated" });
});

app.post("/request/create", async(req, res) => {
    await Request.add(req.body);
    res.send({ msg: "Request created" });
})


app.listen(port, () => {
    console.log(`Servidor levantado en el puerto ${port}`)
});
